<?php
/**
 * Plugin Name: TrustPay Gateway.
 * Plugin URI: https://wordpress.org/plugins/...
 * Description: This plugin enables you to accept online payments for mobile money and cryptocurrencies.
 * Version: 1.1.0
 * Author: Ghana Dot Com
 * Author URI: https://trustpay.africa
 * Author Email: support@ghana.com
 * License: GPLv2 or later
 * Requires at least: 4.4
 * Tested up to: 5.1.1
 * 
 */

				$pluginbase = plugin_dir_url( __FILE__);
				$pluginbasepost = $pluginbase. "postpayment.php";
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))){
    echo "<div class='error notice'><p>Woocommerce has to be installed and active to use the TrustPay Payment Gateway</b> plugin</p></div>";
    return;
}

function trustpay_init()
{
	function add_trustpay_class( $methods ) 
	{
		$methods[] = 'WC_Trustpay_Gateway'; 
		return $methods;
	}
	add_filter( 'woocommerce_payment_gateways', 'add_trustpay_class' );

	if(class_exists('WC_Payment_Gateway'))
	{
		class WC_Trustpay_Gateway extends WC_Payment_Gateway 
		{
			public function __construct()
			{
				$this->id               = 'trustpay';
				$this->icon 			= '../wp-content/plugins/woo-trustpay-payment/pay-logo.jpg';
				$this->has_fields       = false;
				$this->order_button_text = "Proceed to TrustPay";
				$this->method_title     = 'TrustPay Gateway'; 
				$this->method_description =  'Trustpay Standard redirects customers to Trustpay to enter their payment information. For more information and registration, visit <a href = http://trustpay.africa>Trustpay Payment</a>  website';
				$this->description       = 'Easy Payments Using Credit or Debit Cards';            
				$this->init_form_fields();
				$this->init_settings();
				$this->title                    = 'TrustPay';
				$this->trustpay_description       = 'Easy Payments Using Credit or Debit Cards';
				$this->client_key  	    = $this->get_option( 'client_key' );
				$this->client_secret      = $this->get_option( 'client_secret' );
								
				if (is_admin()) 
				{

					if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
						add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
					} else {
						add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
					}				}
				
				//register webhook listener action
				add_action( 'woocommerce_api_wc_trustpay_gateway', array( $this, 'check_trustpay_webhook' ) );
			}
			public function init_form_fields()
			{
				$this->form_fields = array(
					'enabled' => array(
						'title' =>  'Enable/Disable',
						'type' => 'checkbox',
						'label' =>  'Enable TrustPay',
						'default' => 'no'
						),

					'client_key' => array(
						'title' =>  'API Key',
						'type' => 'text',
						'description' =>  "API Key from <a href=https://trustpay.africa target=_blank> Trustpay</a>",
						'default' => '',
						//'desc_tip'      => true,
						'placeholder' => 'API Key'
						
						),

					'client_secret' => array(
						'title' =>  'Client Secret',
						'type' => 'password',
						'description' =>  'Client Secret from <a href=https://trustpay.africa target=_blank> Trustpay</a>',
						'default' => '',
						//'desc_tip'      => true,
						'placeholder' => 'Client Secret'
						),
				);

			}

			/**Function to shrink length of item  */
			protected function limit_length( $string, $limit = 127 ) {
				$str_limit = $limit - 3;
				if ( function_exists( 'mb_strimwidth' ) ) {
					if ( mb_strlen( $string ) > $limit ) {
						$string = mb_strimwidth( $string, 0, $str_limit ) . '...';
					}
				} else {
					if ( strlen( $string ) > $limit ) {
						$string = substr( $string, 0, $str_limit ) . '...';
					}
				}
				return $string;
			}

			/**
			 * handle webhook callback
			 */
			public function check_trustpay_webhook()
			{
				session_start();
				 global $woocommerce;
				 $order_id = $_SESSION['order_id'];
				 
				 $order = new WC_Order( $order_id );
				 
				$status 	= $_POST['result'];
				$rkey 		= $_POST['rkey'];
				$checksum	= $_POST['checksum'];
				$order_tr 	= $_POST['order'];
				$merchant 	= $_POST['merchant'];
				$invoice    = $_POST['invoiceNo'];	

								
			if(isset($_REQUEST['checksum'])  && isset($_REQUEST['rkey']) && isset($_REQUEST['result']) && isset($_REQUEST['order']) && isset($_REQUEST['merchant']))
			
			{
					
					if($status == 'Y') {
						$order->add_order_note('TrustPay Payment Completed');				
							
						//Update the order status
						$order->update_status('payment processed', 'Payment Successful');
						$order->payment_complete($checksum);
	
						//reduce the stock level of items ordered
						wc_reduce_stock_levels($order_id);
						WC()->cart->empty_cart();
						wp_safe_redirect($this->get_return_url($order));						

					}
					else{
					
					$order->add_order_note('Trustpay Payment failed.');
					wp_safe_redirect(wc_get_cart_url($order));
					wc_add_notice('Payment cancelled, Please try again.', 'error');		
				} 
				
			}else {
				wp_safe_redirect(wc_get_cart_url($order));	
				wc_add_notice('Connection failed. Please try again.', 'error');				
			}					
		}
			/**
			 * process payments
			 */
			public function process_payment($order_id)
			{
				session_start();
				global $woocommerce;

				$order = new WC_Order( $order_id );

				// Get an instance of the WC_Order object
				$order = wc_get_order( $order_id );

				$order_data = $order->get_items();

				$pluginbase = plugin_dir_url( __FILE__);
				$pluginbasepost = $pluginbase. "postpayment.php";
				$callbackUrl = WC()->api_request_url( 'WC_Trustpay_Gateway');
				//session_start();
				$_SESSION['first_name']    	= $this->limit_length( $order->get_billing_first_name(), 32 );
				$_SESSION['last_name']     	= $this->limit_length( $order->get_billing_last_name(), 64 );
				$_SESSION['email']		   	= $this->limit_length($order->get_billing_email());
				$_SESSION['total']		   	= $order->get_total();
				$_SESSION['client_key']	   	= $this->client_key;
				$_SESSION['client_secret'] 	= $this->client_secret;
				$_SESSION['redirect_page'] 	= $callbackUrl;
				$_SESSION['order_id'] 	   	= $order_id;
				$_SESSION['code'] 	  	   	= $code;
				$_SESSION['redirect_page'] 	= $callbackUrl;
				$_SESSION['invoice'] 		= $invoice;
				

				$order->update_status('on-hold: awaiting payment', 'Awaiting payment');
							
				return array(
					'result'   => 'success',
					'redirect' => $pluginbasepost
				);
				}					
			}
        }
}

/*Activation hook*/
add_action( 'plugins_loaded', 'trustpay_init' );



